﻿Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

Namespace DomainModel
	Public Interface IDataSourceResetter
		''' <summary>
		''' Reset the Contact data in the database with
		''' entities constructed from sample data
		''' </summary>
		''' <param name="repository">
		''' Repository to during reset. 
		''' This repository also will hold those contacts
		''' after reset.
		''' </param>
		''' <param name="successCallback">Action if reset succeeds.</param>
		''' <param name="errorCallback">Action if it fails.</param>
    Sub ResetContacts(ByVal repository As IContactsRepository, ByVal successCallback As Action(Of IEnumerable(Of Contact)), ByVal errorCallback As Action(Of Exception))
	End Interface

	''' <summary>
	''' Resets the persistent data source (the database) to a clean state
	''' </summary>
	Public Class DataSourceResetter
		Implements IDataSourceResetter

		''' <summary>
		''' Reset the Contact data in the database with
		''' entities constructed from sample data
		''' </summary>
		''' <param name="repository">
		''' Repository to during reset. 
		''' This repository also will hold those contacts
		''' after reset.
		''' </param>
		''' <param name="successCallback">Action if reset succeeds.</param>
		''' <param name="errorCallback">Action if it fails.</param>
    Public Sub ResetContacts(ByVal repository As IContactsRepository, ByVal successCallback As Action(Of IEnumerable(Of Contact)), ByVal errorCallback As Action(Of Exception)) Implements IDataSourceResetter.ResetContacts

      Dim contactsRepository = TryCast(repository, ContactsRepository)

      ' Fish the manager out of the supplied repository
      ' If null repo, create using the current factory method.
      'Dim manager = If(Nothing Is contactsRepository, EntityManagerFactory.Create(), contactsRepository.Manager)
      Dim manager = contactsRepository.Manager

      'Dim op = Coroutine.Start(Function() ResetDataCore(manager))
      'AddHandler op.Completed, Sub(s As Object, e As CoroutineCompletedEventArgs)
      '                           If e.CompletedSuccessfully Then
      '                             manager.FindEntities(Of Contact)(EntityState.AllButDetached).OrderBy(Function(c) c.Id)
      '                             Dim presenter = New Contacts_MVVM.ViewModels.MessagePresenter
      '                             If op.CompletedSuccessfully Then
      '                               presenter.ShowInformationMessage("Datasource reset from sample data")
      '                             Else
      '                               presenter.ShowInformationMessage(op.Error.Message)
      '                             End If
      '                           End If
      '                         End Sub

      Dim op = Coroutine.Start(Function() ResetDataCore(manager))
      AddHandler op.Completed, Sub(s As Object, e As CoroutineCompletedEventArgs)
                                 If e.CompletedSuccessfully Then
                                   successCallback(manager.FindEntities(Of Contact)(EntityState.AllButDetached).OrderBy(Function(c) c.Id))
                                 Else
                                   If (Nothing IsNot errorCallback) Then
                                     e.MarkErrorAsHandled()
                                     errorCallback(e.Error)
                                   End If
                                 End If
                               End Sub


    End Sub


    Private Function ResetDataCore(ByVal manager As IntersoftSampleEntities) As IEnumerable(Of Func(Of INotifyCompleted))

      ' Setup of the list of async functions for the Coroutine to execute serially.
      Dim operationList = New List(Of Func(Of INotifyCompleted))

      manager.QueryCache.Clear() ' forget all prior queries

      Dim f1 As Func(Of INotifyCompleted) =
          Function() manager.Contacts.Include(Contact.EntityPropertyNames.Address).ExecuteAsync()
      operationList.Add(f1)

      Dim f2 As Func(Of INotifyCompleted) =
          Function()
            DeleteAllContacts(manager)
            ContactFactoryFromXml.AddContactsFromSampleDataToManager(manager)
            Return manager.SaveChangesAsync()
          End Function
      operationList.Add(f2)

      Return operationList
    End Function

    Private Shared Sub DeleteAllContacts(ByVal manager As IntersoftSampleEntities)

      ' Delete Contacts we just loaded
      Dim contacts = manager.FindEntities(Of Contact)(EntityState.AllButDetached)

      contacts.ForEach(Sub(entity) entity.EntityAspect.Delete())

      ' Delete Addresses we just loaded
      Dim addrs = manager.FindEntities(Of PostalAddress)(EntityState.AllButDetached)

      addrs.ForEach(Sub(entity) entity.EntityAspect.Delete())
    End Sub

  End Class
End Namespace
