﻿Imports System.Collections
Imports IdeaBlade.EntityModel

Namespace DomainModel

	''' <summary>
	''' Repository interface for creation, deletion, and persistence of Contacts.
	''' </summary>
'INSTANT VB TODO TASK: VB does not allow 'partial' interfaces:
'ORIGINAL LINE: public partial interface IContactsRepository
	Public Interface IContactsRepository
		''' <summary>
		''' Get all Contact graphs asynchronously
		''' </summary>
		''' <param name="onSuccess">Called when get succeeds with load Contacts.</param>
		''' <param name="onFail">Called when get fails with the error.</param>
		''' <remarks>
		''' Order of results is not determined; 
		''' QueryStrategy is unspecified;
		''' The "get" verb implies (but does not demand)
		''' that the implementation will retrieve the
		''' contacts from the database unless they have been "gotten" before.
		''' </remarks>
    Sub GetContacts(ByVal onSuccess As Action(Of IEnumerable(Of Contact)), ByVal onFail As Action(Of Exception))

		''' <summary>
		''' Save all pending changes (add/update/delete)
		''' </summary>
		''' <param name="onSuccess">Called with entities actually saved when save succeeds</param>
		''' <param name="onFail">
		''' Called when fails with the error.
		''' if no errors, you may infer that save was cancelled;
		''' check the returned operation object if you doubt it.
		''' </param>
    'Sub Save(Optional ByVal onSuccess As Action(Of IEnumerable) = Nothing, Optional ByVal onFail As Action(Of Exception) = Nothing)
    Sub Save(ByVal onSuccess As Action(Of System.Collections.IEnumerable), ByVal onFail As Action(Of Exception))

		''' <summary>
		''' Rollback all pending changes in the repository.
		''' </summary>
		Sub Rollback()

		''' <summary>
		''' Return newly created Contact graph and add it to the repository
		''' </summary>
		Function CreateContact() As Contact

		''' <summary>
		''' Schedule deletion of the Contact graph, pending save.
		''' </summary>
		Sub DeleteContact(ByVal contact As Contact)
	End Interface

End Namespace