﻿Imports DomainModel
Imports Intersoft.Client.Framework

Namespace Contacts_MVVM.ViewModels
	''' <summary>
	''' DialogBox Contact Editor for editing a new or existing contact.
	''' </summary>
	''' <remarks>
	''' Opens an editor in a DialogBox at the request of the parent
	''' "list-of-Contacts" view, <see cref="ContactsViewModel"/>. 
	''' .
	''' <para>
	''' Several methods are virtual and/or protected to support
	''' the <see cref="ContactSandboxEditor"/> that derives from it.
	''' </para>
	''' This implementation uses the "provider" pattern
	''' to open the view (contact dialog box),
	''' <see cref="Contacts_MVVM.Views.ContactEditDialogBox"/>,
	''' and associate it with its ViewModel,
	''' <see cref="ContactEditViewModel"/>.
	''' </remarks>
	Public Class ContactEditor
		Implements IContactEditor
		Public Sub New(ByVal repository As IContactsRepository)
			Me.Repository = repository
		End Sub
    Public Property conlViewModel As Contacts_MVVM.ViewModels.ContactsViewModel

		Protected Property Repository() As IContactsRepository

		Public Overridable Sub EditExistingContact(ByVal sourceContact As Contact, ByVal onOk As Action(Of Contact), ByVal onCancel As Action) Implements IContactEditor.EditExistingContact
			ShowContactDialogBox(sourceContact, onOk, onCancel)
		End Sub

		Public Overridable Sub AddNewContact(ByVal onOk As Action(Of Contact), ByVal onCancel As Action) Implements IContactEditor.AddNewContact
			Dim contact = Repository.CreateContact()
			ShowContactDialogBox(contact, onOk, onCancel)
		End Sub

		Protected Sub ShowContactDialogBox(ByVal contact As Contact, ByVal onOk As Action(Of Contact), ByVal onCancel As Action)
					' show the Edit Contact dialog box (view)
			DialogBoxServiceProvider.GetInstanceAsync(DialogBoxServiceProvider.ContactEditView, Sub(dialogBox)
				Dim viewModel = GetViewModel(contact)
				dialogBox.DataContext = viewModel
				DialogBoxServiceProvider.Show(dialogBox, Sub(dialogResult) OnContactDialogBoxClosed(dialogResult, viewModel, onOk, onCancel))
			End Sub)
		End Sub

		Protected Overridable Sub OnContactDialogBoxClosed(ByVal dialogResult As DialogResult, ByVal viewModel As ContactEditViewModel, ByVal onOk As Action(Of Contact), ByVal onCancel As Action)
      If dialogResult = dialogResult.OK Then
        ' user clicked save button
        Dim contact = GetSavedContact(viewModel)
        If onOk IsNot Nothing Then
          onOk(contact)
        End If
      Else
        ' user clicked cancel button or close box
        RollbackChanges()
        viewModel.Cancel()
        If onCancel IsNot Nothing Then
          onCancel()
        End If
      End If
		End Sub

		Private Function GetViewModel(ByVal contact As Contact) As ContactEditViewModel
			Return New ContactEditViewModel(contact, Repository, AddressOf DialogBoxServiceProvider.CloseCurrentWindow)
		End Function

		Protected Overridable Sub RollbackChanges()
			' Execute rollback in the next thread
			' to ensure UI and binding handlers have completed
			CrossPlatform.BeginInvoke(New Action(AddressOf Repository.Rollback))
		End Sub

		Protected Overridable Function GetSavedContact(ByVal viewModel As ContactEditViewModel) As Contact
			Return viewModel.Contact
		End Function

	End Class
End Namespace