Imports DomainModel

Namespace Contacts_MVVM.ViewModels
	''' <summary>
	''' A non-production version of the <see cref="ContactEditViewModel"/>
	''' for displaying sample data in visual design tools 
	''' such as "Cider" and Blend.
	''' </summary>
	''' <remarks>
	''' Goal #1: enable view layout using design-time data.
	''' Goal #2: minimize discrepancy between design-time data and runtime data.
	''' <para>
	''' In this approach, the View xaml specifies a design-time DataContext as in
	''' <![CDATA[
	''' d:DataContext="{d:DesignInstance Type=designViewModels:DesignContactEditViewModel, IsDesignTimeCreatable=True}"
	''' ]]>
	''' </para><para>
	''' Can't use the real <see cref="ContactEditViewModel"/>
	''' because its constructor takes parameters, specifically the
	''' Contact that is the subject of the View.
	''' Xaml can only create objects with parameter-less constructors.
	''' Therefore, this "Design" ViewModel both presents a parameterless-ctor
	''' and provides a design-time Contact that it creates from sample data.
	''' <para></para>
	''' It also derives from the production ViewModel so that the view,
	''' when seen with design tools, binds to the same
	''' "surface" of properties and commands as it will encounter
	''' in the production ViewModel.
	''' </para>
	''' </remarks>
	Public Class DesignContactEditViewModel
		Inherits ContactEditViewModel
		''' <summary>
		''' Create design-time ContactEditViewModel
		''' </summary>
		''' <remarks>
		''' No need for a Repository or "CloseEdit" action 
		''' when working within visual design tools.
		''' </remarks>
		Public Sub New()
			MyBase.New(GetDesignContact(), Nothing, Nothing)
		End Sub

		Private Shared Function GetDesignContact() As Contact
			Return ContactFactoryFromXml.GetContactsFromSampleData().First(Function(c) c.Id = "CARIN")
		End Function
	End Class
End Namespace