﻿Imports System.ComponentModel.DataAnnotations
Imports IdeaBlade.Core.ComponentModel.DataAnnotations ' Silverlight
Imports IdeaBlade.EntityModel
Imports IdeaBlade.Validation
Imports DomainModel.Contact

''' <summary>
''' Extension of generated Contact class with custom properties 
''' and validation metadata.
''' Also able to validate itself and its related Address entity;
''' signals that ability by inheriting from <see cref="ISelfValidator"/>. 
''' </summary>

Namespace DomainModel
  <MetadataType(GetType(ContactMetadata))>
  Partial Public Class Contact
    Implements ISelfValidator

#Region "Custom Properties"

    ' Reset ContactPhotoFinder in application with a real version
    ' See ContactsRepository
    Public Shared ContactPhotoFinder As IContactPhotoFinder = New NullPhotoFinder()

    'For demo simplicity, get photos based on Id
    Public ReadOnly Property Photo() As Uri
      Get
        Return ContactPhotoFinder.GetPhotoUriFromId(Me.Id)
      End Get
    End Property


    Public Overrides Function ToString() As String
      Return String.Format("Contact ({0}) {1}", ContactID, Name)
    End Function

#End Region

#Region "buddy class"

    Public NotInheritable Class ContactMetadata
      <Required()>
      Public Shared Address As PostalAddress

      ' Phone numbers

      ' Only "Phone" is required; the rest are optional
      <PhoneNumberVerifier("Phone", IsRequired:=True)>
      Public Shared Phone As String

      <PhoneNumberVerifier("Fax")>
      Public Shared Fax As String

      <PhoneNumberVerifier("Cell")>
      Public Shared Cell As String

      ' Uri's - all optional

      <UriVerifier("Blog")>
      Public Shared Blog As String

      <UriVerifier("Twitter")>
      Public Shared Twitter As String

      <UriVerifier("Website")>
      Public Shared Website As String
    End Class

#End Region

#Region "ISelfValidator"

    Public Function Validate() As Boolean Implements ISelfValidator.Validate
      If Me.EntityAspect.EntityManager Is Nothing Then
        Return True
      End If

      Return Validate(Me.EntityAspect.EntityManager.VerifierEngine, Me).Ok
    End Function

    Public Function Validate(ByVal verifierEngine As VerifierEngine, ByVal verifierContext As Object) As VerifierResultCollection Implements ISelfValidator.Validate
      ' Don't bother validating this entity if we're deleting it
      If Me.EntityAspect.EntityState = EntityState.Deleted Then
        Return New VerifierResultCollection() ' empty is always OK
      End If

      If verifierEngine Is Nothing Then
        Throw New ArgumentNullException("verifierEngine")
      End If

      Dim contactResults = verifierEngine.Execute(Me)
      Dim addrResults = verifierEngine.Execute(Me.Address)
      NotifyThatValidationChanged()
      Return New VerifierResultCollection(contactResults.Union(addrResults))
    End Function

    Private Sub NotifyThatValidationChanged()
      ' Workaround for WPF and limitations of IDataErrorInfo
      ' Raises PropertyChanged for the benefit of IDataErrorInfo consumers (WPF)
      ' INotifyDataErrorInfo consumers (Silverlight) don't need PropertyChanged;
      ' that interface has its own notification event.

      ' Raising with null means "update all of your bindings"
      OnPropertyChanged(New System.ComponentModel.PropertyChangedEventArgs(Nothing))
    End Sub
#End Region

  End Class

#Region "ContactPhotoFinder"

  Public Interface IContactPhotoFinder
    Function GetPhotoUriFromId(ByVal id As String) As Uri
  End Interface

  Friend Class NullPhotoFinder
    Implements IContactPhotoFinder
    Public Function GetPhotoUriFromId(ByVal id As String) As Uri Implements IContactPhotoFinder.GetPhotoUriFromId
      Return Nothing
    End Function
  End Class

#End Region

End Namespace