﻿Imports System.ComponentModel.DataAnnotations
Imports IdeaBlade.Core.ComponentModel.DataAnnotations ' Silverlight
Imports IdeaBlade.Validation
Imports DomainModel.PostalAddress
Imports DomainModel.Contact

Namespace DomainModel
	''' <summary>
	''' Extension of generated PostalAddress class with custom properties 
	''' and validation metadata.
	''' </summary>
  <MetadataType(GetType(PostalAddressMetadata))>
  Partial Public Class PostalAddress

#Region "Custom Properties"

    Public Const DefaultStateCode As StateCode = DomainModel.StateCode.CA

    <RequiredValueVerifier()>
    Public Property State() As StateCode
      Get
        Return GetStateCodeFromStateCodeName()
      End Get
      Set(ByVal value As StateCode)
        Me.StateCodeName = value.ToString()
      End Set
    End Property

    Private Function GetStateCodeFromStateCodeName() As StateCode
      Try
        Return CType(System.Enum.Parse(GetType(StateCode), Me.StateCodeName, True), StateCode)
      Catch e1 As Exception
        ' "Fix" the StateCode while coercing to default value
        ' Do not set the State as it caused the object to dirty
        ' which is not desired in "new" mode
        Return DefaultStateCode
      End Try
    End Function

    Public Overrides Function ToString() As String
      Dim contact = Me.Contacts.FirstOrDefault()
      Dim name = If(Nothing Is contact, "<no contact>", contact.Name)
      Return String.Format("PostalAddress ({0}) for {1}", AddressID, name)
    End Function

#End Region

#Region "buddy class"

    Public NotInheritable Class PostalAddressMetadata

      <RegexVerifier("Zip Code", "^[0-9]{5}(-[0-9]{4})?$", ErrorMessage:="Please specify a valid zip code.")>
      Public Shared ZipCode As String
    End Class
#End Region
  End Class

	Public Enum StateCode
		AL ' Alabama
		AK ' Alaska
		[AS] ' American Samoa
		AZ ' Arizona
		AR ' Arkansas
		CA ' California
		CO ' Colorado
		CT
		DE
		DC
		FL
		GA
		GU
		HI
		ID
		IL
		[IN]
		IA
		KS
		KY
		LA
		[ME]
		MD
		MH
		MA
		MI
		FM
		MN
		MS
		MO
		MT
		NE
		NV
		NH
		NJ
		NM
		NY
		NC
		ND
		MP
		OH
		OK
		[OR]
		PW
		PA
		PR ' Puerto Rico
		RI
		SC
		SD
		TN
		TX
		UT
		VT
		VA
		VI
		WA
		WV
		WI
		WY
	End Enum

End Namespace
