﻿Imports IdeaBlade.Validation

Namespace DomainModel

	'''* Application-specific validation support classes and interfaces **

	''' <summary>
	''' PhoneNumber validation attribute
	''' </summary>
	Public Class PhoneNumberVerifier
		Inherits RegexVerifierAttribute
		Public Sub New(ByVal displayName As String)
			MyBase.New(displayName, RegExSpec)
			ErrorMessage = "Please specify a valid US phone number."
		End Sub

		Private Const RegExSpec As String = "^(1-?)?(?<areacode>[0-9]{3})-?(?<prefix>[0-9]{3})-?(?<suffix>[0-9]{4})$"
	End Class

	''' <summary>
	''' URI address validation attribute
	''' </summary>
	Public Class UriVerifier
		Inherits RegexVerifierAttribute
		Public Sub New(ByVal displayName As String)
			MyBase.New(displayName, RegExSpec)
			ErrorMessage = "Please specify a valid Url."
		End Sub

		Private Const RegExSpec As String = "^(?#Protocol)(?:(?:ht|f)tp(?:s?)\:\/\/|~\/|\/)?(?#Username:Password)(?:\w+:\w+@)?(?#Subdomains)(?:(?:[-\w]+\.)+(?#TopLevel Domains)(?:com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|museum|travel|[a-z]{2}))(?#Port)(?::[\d]{1,5})?(?#Directories)(?:(?:(?:\/(?:[-\w~!$+|.,=]|%[a-f\d]{2})+)+|\/)+|\?|#)?(?#Query)(?:(?:\?(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)(?:&(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)*)*(?#Anchor)(?:#(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)?$"
	End Class

	''' <summary>
	''' Implementors of this interface can validate themselves.
	''' </summary>
	''' <remarks>
	''' See <see cref="Contact"/>
	''' </remarks>
	Public Interface ISelfValidator
		Function Validate() As Boolean
		Function Validate(ByVal verifierEngine As VerifierEngine, ByVal verifierContext As Object) As VerifierResultCollection
	End Interface

End Namespace